<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: route_guide.proto

namespace Routeguide;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * A latitude-longitude rectangle, represented as two diagonally opposite
 * points "lo" and "hi".
 *
 * Generated from protobuf message <code>routeguide.Rectangle</code>
 */
class Rectangle extends \Google\Protobuf\Internal\Message
{
    /**
     * One corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point lo = 1;</code>
     */
    protected $lo = null;
    /**
     * The other corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point hi = 2;</code>
     */
    protected $hi = null;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type \Routeguide\Point $lo
     *           One corner of the rectangle.
     *     @type \Routeguide\Point $hi
     *           The other corner of the rectangle.
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\RouteGuide::initOnce();
        parent::__construct($data);
    }

    /**
     * One corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point lo = 1;</code>
     * @return \Routeguide\Point|null
     */
    public function getLo()
    {
        return isset($this->lo) ? $this->lo : null;
    }

    public function hasLo()
    {
        return isset($this->lo);
    }

    public function clearLo()
    {
        unset($this->lo);
    }

    /**
     * One corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point lo = 1;</code>
     * @param \Routeguide\Point $var
     * @return $this
     */
    public function setLo($var)
    {
        GPBUtil::checkMessage($var, \Routeguide\Point::class);
        $this->lo = $var;

        return $this;
    }

    /**
     * The other corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point hi = 2;</code>
     * @return \Routeguide\Point|null
     */
    public function getHi()
    {
        return isset($this->hi) ? $this->hi : null;
    }

    public function hasHi()
    {
        return isset($this->hi);
    }

    public function clearHi()
    {
        unset($this->hi);
    }

    /**
     * The other corner of the rectangle.
     *
     * Generated from protobuf field <code>.routeguide.Point hi = 2;</code>
     * @param \Routeguide\Point $var
     * @return $this
     */
    public function setHi($var)
    {
        GPBUtil::checkMessage($var, \Routeguide\Point::class);
        $this->hi = $var;

        return $this;
    }

}

